#!/usr/bin/env python
import sys
import mysql.connector
import rosapi
import settings

class Rule(object):
	def __init__(self, d):
		self.id = d['id']
		self.chain = d['chain']
		self.protocol = d['protocol']
		self.source_ip = d['src-address']
		self.action = d['action']

	def __str__(self):
		return '{}: proto {} {} {}'.format(self.id, self.protocol, self.action, self.source_ip)

class Device(object):
	def __init__(self, hostname, software_id, **kwargs):
		self.hostname = hostname
		self.software_id = software_id
		for key, value in kwargs.iteritems():
			setattr(self, key, value)

class MikrotikDevice(Device):
	def __init__(self, hostname, software_id, **kwargs):
		super(MikrotikDevice, self).__init__(hostname, software_id, **kwargs)
		self.api = rosapi.RouterboardAPI(self.hostname, self.username, self.password)

	def sync(self, filter_rules):
		print "==> Syncing device {}".format(self.hostname)

		# check software-id
		r = self.api.get_resource('/system/license').call('print', {})[0]
		if not 'software-id' in r:
			sys.exit("Error: Host `{}': can not obtain software-id".format(self.hostname))
		software_id = r['software-id']
		print 'software-id: {}'.format(software_id)
		if software_id != self.software_id:
			sys.exit("Error: Host `{}': software-id mismatch: {} (expected {})".format(self.hostname, software_id, self.software_id))

		# obtain the list of existing rules
		existing_rules = {}
		for r in self.api.get_resource('/ip/firewall/filter').call('print', {}):
			if not 'chain' in r or not 'protocol' in r or not 'src-address' in r:
				continue
			rule = Rule(r)
			print "--> rule {}".format(rule)
			if not rule.protocol == 'udp':
				continue
			if not rule.source_ip in filter_rules:
				continue
			if rule.action != 'accept' and filter_rules[rule.source_ip] == 'allow':
				self.delete_rule(rule)
				continue
			if rule.action == 'accept' and filter_rules[rule.source_ip] != 'allow':
				self.delete_rule(rule)
				continue
			existing_rules[rule.source_ip] = rule

		for ip in filter_rules.keys():
			if ip in existing_rules:
				continue
			if filter_rules[ip] == 'allow':
				continue
			rule = Rule(dict({ 'id': None, 'chain': 'input', 'protocol': 'udp', 'src-address': ip, 'action': 'drop' }))
			print "--> rule {}".format(rule)
			self.add_rule(rule)
			
	def delete_rule(self, rule):
		print "\tdeleting"
		self.api.get_resource('/ip/firewall/filter').call('remove', { 'id': rule.id })

	def add_rule(self, rule):
		print "\tadding"
		params = {
			'chain': 'input',
			'protocol': 'udp',
			'src-address': rule.source_ip,
			'action': rule.action,
			}
		self.api.get_resource('/ip/firewall/filter').call('add', params)

# filter rules ip -> action
filter_rules = {}

cnx = mysql.connector.connect(**settings.DBCONFIG)
cur = cnx.cursor()
cur.execute('SELECT source_ip, action FROM filter_rules')
for source_ip, action in cur:
	if source_ip in filter_rules:
		# 'allow' has the least precedence (never supersedes other)
		# 'deny' has highest precedence (never is superseded)
		if action == 'allow' or filter_rules[source_ip] == 'deny': 
			continue
	print "--> {}: {}".format(source_ip, action)
	filter_rules[source_ip] = action

# fetch the list of registered (software_id is not NULL) systems
cur.execute('SELECT hostname, software_id, username, password FROM devices WHERE software_id IS NOT NULL')
for hostname, software_id, username, password in cur:
	d = MikrotikDevice(hostname, software_id, username=username, password=password)
	d.sync(filter_rules)
